@@ -21,7 +21,7 @@ static inline long __get_page_size(void)
 		ssize_t x;
 
 		x = __sys_read(fd, buf, sizeof(buf));
-		if (x < sizeof(buf))
+		if (x < (long) sizeof(buf))
 			break;
 
 		if (buf[0] == AT_PAGESZ) {
